@@ -1,19 +1,21 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2019-2020 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2019-2022 -- leonerd@leonerd.org.uk
 
-package Object::Pad;
+package Object::Pad 0.61;
 
 use v5.14;
 use warnings;
 
-our $VERSION = '0.36';
-
 use Carp;
 
-require XSLoader;
-XSLoader::load( __PACKAGE__, $VERSION );
+sub dl_load_flags { 0x01 }
+
+require DynaLoader;
+__PACKAGE__->DynaLoader::bootstrap( our $VERSION );
+
+our $XSAPI_VERSION = "0.48";
 
 # So that feature->import will work in `class`
 require feature;
@@ -24,23 +26,45 @@ if( $] >= 5.020 ) {
 
 require mro;
 
+require Object::Pad::MOP::Class;
+
 =head1 NAME
 
-C<Object::Pad> - a simple syntax for lexical slot-based objects
+C<Object::Pad> - a simple syntax for lexical field-based objects
 
 =head1 SYNOPSIS
 
+On perl version 5.26 onwards:
+
+   use v5.26;
+   use Object::Pad;
+
+   class Point {
+      has $x :param = 0;
+      has $y :param = 0;
+
+      method move ($dX, $dY) {
+         $x += $dX;
+         $y += $dY;
+      }
+
+      method describe () {
+         print "A point at ($x, $y)\n";
+      }
+   }
+
+   Point->new(x => 5, y => 10)->describe;
+
+Or, for older perls that lack signatures:
+
    use Object::Pad;
 
    class Point {
-      has $x = 0;
-      has $y = 0;
+      has $x :param = 0;
+      has $y :param = 0;
 
-      BUILD {
-        ($x, $y) = @_;
-      }
-
-      method move ($dX, $dY) {
+      method move {
+         my ($dX, $dY) = @_;
          $x += $dX;
          $y += $dY;
       }
@@ -50,20 +74,19 @@ C<Object::Pad> - a simple syntax for lexical slot-based objects
       }
    }
 
-   Point->new(5,10)->describe;
+   Point->new(x => 5, y => 10)->describe;
 
 =head1 DESCRIPTION
 
 This module provides a simple syntax for creating object classes, which uses
 private variables that look like lexicals as object member fields.
 
-B<WARNING> This module is still very experimental. The parts that currently
-exist do seem to work reliably but much of the design is still evolving, and
-many features and have yet to be implemented. I don't yet guarantee I won't
-have to change existing details in order to continue its development. Feel
-free to try it out in experimental or newly-developed code, but don't complain
-if a later version is incompatible with your current code and you'll have to
-change it.
+B<WARNING> This module is still experimental. The parts that currently exist
+do seem to work reliably but much of the design is still evolving, and many
+features and have yet to be implemented. I don't yet guarantee I won't have to
+change existing details in order to continue its development. Feel free to try
+it out in experimental or newly-developed code, but don't complain if a later
+version is incompatible with your current code and you'll have to change it.
 
 That all said, please do get in contact if you find the module overall useful.
 The more feedback you provide in terms of what features you are using, what
@@ -73,12 +96,11 @@ hopefully eventual stability of the design. See the L</FEEDBACK> section.
 =head2 Automatic Construction
 
 Classes are automatically provided with a constructor method, called C<new>,
-which helps create the object instances.
+which helps create the object instances. This may respond to passed arguments,
+automatically assigning values of fields, and invoking other blocks of code
+provided by the class. It proceeds in the following stages:
 
-As part of the construction process, the C<BUILD> block of every component
-class will be invoked, passing in the list of arguments the constructor was
-invoked with. Each class should perform its required setup behaviour, but does
-not need to chain to the C<SUPER> class first; this is handled automatically.
+=head3 The BUILDARGS phase
 
 If the class provides a C<BUILDARGS> class method, that is used to mangle the
 list of arguments before the C<BUILD> blocks are called. Note this must be a
@@ -87,6 +109,42 @@ should perform any C<SUPER> chaining as may be required.
 
    @args = $class->BUILDARGS( @_ )
 
+=head3 Field assignment
+
+If any field in the class has the C<:param> attribute, then the constructor
+will expect to receive its argmuents in an even-sized list of name/value
+pairs. This applies even to fields inherited from the parent class or applied
+roles. It is therefore a good idea to shape the parameters to the constructor
+in this way in roles, and in classes if you intend your class to be extended.
+
+The constructor will also check for required parameters (these are all the
+parameters for fields that do not have default initialisation expressions). If
+any of these are missing an exception is thrown.
+
+=head3 The BUILD phase
+
+As part of the construction process, the C<BUILD> block of every component
+class will be invoked, passing in the list of arguments the constructor was
+invoked with. Each class should perform its required setup behaviour, but does
+not need to chain to the C<SUPER> class first; this is handled automatically.
+
+=head3 The ADJUST phase
+
+Next, the C<ADJUST> and C<ADJUSTPARAMS> block of every component class is
+invoked. This happens after the fields are assigned their initial values and
+the C<BUILD> blocks have been run.
+
+Note also that both C<ADJUST> and C<ADJUSTPARAMS> blocks happen at the same
+time, in declaration order. The C<ADJUSTPARAMS> blocks do not form their own
+separate phase.
+
+=head3 The strict-checking phase
+
+Finally, before the object is returned, if the L</:strict(params)> class
+attribute is present, then the constructor will throw an exception if there
+are any remaining named arguments left over after assigning them to fields as
+per C<:param> declarations, and running any C<ADJUSTPARAMS> blocks.
+
 =head1 KEYWORDS
 
 =head2 class
@@ -113,20 +171,65 @@ sets the value of the package's C<$VERSION> variable.
 
    class Name VERSION;
 
-A single superclass is supported by the keyword C<extends>
+A single superclass is supported by the keyword C<isa>
 
-   class Name extends BASECLASS {
+I<Since version 0.41.>
+
+   class Name isa BASECLASS {
       ...
    }
 
-   class Name extends BASECLASS BASEVER {
+   class Name isa BASECLASS BASEVER {
       ...
    }
 
-If a package providing the superclass does not exist, an attempt is made to
+Prior to version 0.41 this was called C<extends>, which is currently
+recognised as a compatibility synonym. Both C<extends> and C<isa> keywords are
+now discouraged, in favour of the L</:isa> attribute which is preferred
+because it follows a more standard grammar without this special-case.
+
+One or more roles can be composed into the class by the keyword C<does>
+
+I<Since version 0.41.>
+
+   class Name does ROLE, ROLE,... {
+      ...
+   }
+
+Prior to version 0.41 this was called C<implements>, which is currently
+recognised as a compatibility synonym. Both C<implements> and C<does> keywords
+are now discouraged, in favour of the L</:does> attribute which is preferred
+because it follows a more standard grammar without this special-case.
+
+An optional list of attributes may be supplied in similar syntax as for subs
+or lexical variables. (These are annotations about the class itself; the
+concept should not be confused with per-object-instance data, which here is
+called "fields").
+
+Whitespace is permitted within the value and is automatically trimmed, but as
+standard Perl parsing rules, no space is permitted between the attribute's
+name and the open parenthesis of its value:
+
+   :attr( value here )     # is permitted
+   :attr (value here)      # not permitted
+
+The following class attributes are supported:
+
+=head3 :isa
+
+   :isa(CLASS)
+
+   :isa(CLASS CLASSVER)
+
+I<Since version 0.57.>
+
+Declares a superclass that this class extends. At most one superclass is
+supported.
+
+If the package providing the superclass does not exist, an attempt is made to
 load it by code equivalent to
 
-   require Animal ();
+   require CLASS ();
 
 and thus it must either already exist, or be locatable via the usual C<@INC>
 mechanisms.
@@ -139,16 +242,23 @@ An optional version check can also be supplied; it performs the equivalent of
 
    BaseClass->VERSION( $ver )
 
-One or more roles can be composed into the class by the keyword C<implements>
+=head3 :does
 
-   class Name implements ROLE, ROLE,... {
-      ...
-   }
+   :does(ROLE)
 
-An optional list of attributes may be supplied in similar syntax as for subs
-or lexical variables. (These are annotations about the class itself; the
-concept should not be confused with per-object-instance data, which here is
-called "slots"). The following class attributes are supported:
+   :does(ROLE ROLEVER)
+
+I<Since version 0.57.>
+
+Composes a role into the class; optionally requiring a version check on the
+role package. This is a newer form of the C<implements> and C<does>
+keywords and should be preferred for new code.
+
+Multiple roles can be composed by using multiple C<:does> attributes, one per
+role.
+
+The package will be loaded in a similar way to how the L</:isa> attribute is
+handled.
 
 =head3 :repr(TYPE)
 
@@ -199,6 +309,24 @@ This achieves the best combination of DWIM while still allowing the common
 forms of hash reference to be inspected by C<Data::Dumper>, etc. This is the
 default representation type, and does not have to be specifically requested.
 
+=head3 :strict(params)
+
+I<Since version 0.43.>
+
+Can only be applied to classes that contain no C<BUILD> blocks. If set, then
+the constructor will complain about any unrecognised named arguments passed to
+it (i.e. names that do not correspond to the C<:param> of any defined field
+and left unconsumed by any C<ADJUSTPARAMS> block).
+
+Since C<BUILD> blocks can inspect the arguments arbitrarily, the presence of
+any such block means the constructor cannot determine which named arguments
+are not recognised.
+
+This attribute is a temporary stepping-stone for compatibility with existing
+code. It is recommended to enable this whenever possible, as a later version
+of this module will likely perform this behaviour unconditionally whenever no
+C<BUILD> blocks are present.
+
 =head2 role
 
    role Name :ATTRS... {
@@ -229,19 +357,28 @@ implements the role.
       requires METHOD;
    }
 
-A role can provide instance slots. These are visible to any C<BUILD> blocks or
-methods provided by that role.
+A role can provide instance fields. These are visible to any C<BUILD> blocks
+or methods provided by that role.
 
 I<Since version 0.33.>
 
    role Name {
-      has $slot;
+      has $field;
 
-      BUILD { $slot = "a value" }
+      BUILD { $field = "a value" }
 
-      method slot { return $slot }
+      method field { return $field }
    }
 
+I<Since version 0.57> a role can declare that it provides another role:
+
+   role Name :does(OTHERROLE) { ... }
+   role Name :does(OTHERROLE OTHERVER) { ... }
+
+This will include all of the methods from the included role. Effectively this
+means that applying the "outer" role to a class will imply applying the other
+role as well.
+
 The following role attributes are supported:
 
 =head3 :compat(invokable)
@@ -263,7 +400,7 @@ allows the creation of a role that can still provide code for existing classes
 written in classical Perl that has not yet been rewritten to use
 C<Object::Pad>.
 
-The tradeoff is that a C<:compat(invokable)> role may not create slot data
+The tradeoff is that a C<:compat(invokable)> role may not create field data
 using the L</has> keyword. Whatever behaviours the role wishes to perform
 must be provided only by calling other methods on C<$self>, or perhaps by
 making assumptions about the representation type of instances.
@@ -276,59 +413,96 @@ the role.
 =head2 has
 
    has $var;
-   has $var = EXPR;
    has @var;
    has %var;
 
    has $var :ATTR ATTR...;
 
+   has $var = EXPR;
+
+   has $var { BLOCK };
+
 Declares that the instances of the class or role have a member field of the
-given name. This member field (called a "slot") will be accessible as a
-lexical variable within any C<method> declarations in the class.
+given name. This member field will be accessible as a lexical variable within
+any C<method> declarations in the class.
 
 Array and hash members are permitted and behave as expected; you do not need
 to store references to anonymous arrays or hashes.
 
 Member fields are private to a class or role. They are not visible to users of
-the class, nor to subclasses, nor to any class that a role is applied to. In
-order to provide access to them a class may wish to use L</method> to create
-an accessor, or use the attributes such as L</:reader> to get one generated.
+the class, nor inherited by subclasses nor any class that a role is applied
+to. In order to provide access to them a class may wish to use L</method> to
+create an accessor, or use the attributes such as L</:reader> to get one
+generated.
 
-A scalar slot may provide a expression that gives an initialisation value,
-which will be assigned into the slot of every instance during the constructor
+A scalar field may provide a expression that gives an initialisation value,
+which will be assigned into the field of every instance during the constructor
 before the C<BUILD> blocks are invoked. I<Since version 0.29> this expression
 does not have to be a compiletime constant, though it is evaluated exactly
 once, at runtime, after the class definition has been parsed. It is not
-evaluated individually for every object instance of that class.
+evaluated individually for every object instance of that class. I<Since
+version 0.54> this is also permitted on array and hash fields.
 
-The following slot attributes are supported:
+=head3 Field Initialiser Blocks
+
+I<Since version 0.54> a deferred statement block is also permitted, on any
+field variable type. This is an B<experimental> feature that permits code to
+be executed as part of the instance constructor, rather than running just
+once when the class is set up. Code in a field initialisation block is
+roughly equivalent to being placed in a C<BUILD> or C<ADJUST> block.
+
+Control flow that attempts to leave a field initialiser block is not
+permitted. This includes any C<return> expression, any C<next/last/redo>
+outside of a loop, with a dynamically-calculated label expression, or with a
+label that it doesn't appear in. C<goto> statements are also currently
+forbidden, though known-safe ones may be permitted in future.
+
+Loop control expressions that are known at compiletime to affect a loop that
+they appear within are permitted.
+
+   has $field { foreach(@list) { next; } }       # this is fine
+
+   has $field { LOOP: while(1) { last LOOP; } }  # this is fine too
+
+The following field attributes are supported:
 
 =head3 :reader, :reader(NAME)
 
 I<Since version 0.27.>
 
-Generates a reader method to return the current value of the slot. Currently
-these are only permitted for scalar slots. If no name is given, the name of
-the slot is used. A single prefix character C<_> will be removed if present.
+Generates a reader method to return the current value of the field. If no name
+is given, the name of the field is used. A single prefix character C<_> will
+be removed if present.
 
-   has $slot :reader;
+   has $field :reader;
 
    # equivalent to
-   has $slot;  method slot { return $slot }
+   has $field;  method field { return $field }
+
+I<Since version 0.55> these are permitted on any field type, but prior
+versions only allowed them on scalar fields. The reader method behaves
+identically to how a lexical variable would behave in the same context; namely
+returning a list of values from an array or key/value pairs from a hash when
+in list context, or the number of items or keys when in scalar context.
+
+   has @items :reader;
+
+   foreach my $item ( $obj->items ) { ... }   # iterates the list of items
+
+   my $count = $obj->items;                   # yields count of items
 
 =head3 :writer, :writer(NAME)
 
 I<Since version 0.27.>
 
-Generates a writer method to set a new value of the slot from its first
-argument. Currently these are only permitted for scalar slots. If no name is
-given, the name of the slot is used prefixed by C<set_>. A single prefix
-character C<_> will be removed if present.
+Generates a writer method to set a new value of the field from its arguments.
+If no name is given, the name of the field is used prefixed by C<set_>. A
+single prefix character C<_> will be removed if present.
 
-   has $slot :writer;
+   has $field :writer;
 
    # equivalent to
-   has $slot;  method set_slot { $slot = shift; return $self }
+   has $field;  method set_field { $field = shift; return $self }
 
 I<Since version 0.28> a generated writer method will return the object
 invocant itself, allowing a chaining style.
@@ -337,24 +511,90 @@ invocant itself, allowing a chaining style.
       ->set_y("y")
       ->set_z("z");
 
+I<Since version 0.55> these are permitted on any field type, but prior
+versions only allowed them on scalar fields. On arrays or hashes, the writer
+method takes a list of values to be assigned into the field, completely
+replacing any values previously there.
+
 =head3 :mutator, :mutator(NAME)
 
 I<Since version 0.27.>
 
-Generates an lvalue mutator method to return or set the value of the slot.
-These are only permitted for scalar slots. If no name is given, the name of
-the slot is used. A single prefix character C<_> will be removed if present.
+Generates an lvalue mutator method to return or set the value of the field.
+These are only permitted for scalar fields. If no name is given, the name of
+the field is used. A single prefix character C<_> will be removed if present.
 
-   has $slot :mutator;
+   has $field :mutator;
 
    # equivalent to
-   has $slot;  method slot :lvalue { $slot }
+   has $field;  method field :lvalue { $field }
 
 I<Since version 0.28> all of these generated accessor methods will include
 argument checking similar to that used by subroutine signatures, to ensure the
 correct number of arguments are passed - usually zero, but exactly one in the
 case of a C<:writer> method.
 
+=head3 :accessor, :accessor(NAME)
+
+I<Since version 0.53.>
+
+Generates a combined reader-writer accessor method to set or return the value
+of the field. These are only permitted for scalar fields. If no name is given,
+the name of the field is used. A prefix character C<_> will be removed if
+present.
+
+This method takes either zero or one additional arguments. If an argument is
+passed, the value of the field is set from this argument (even if it is
+C<undef>). If no argument is passed (i.e. C<scalar @_> is false) then the
+field is not modified. In either case, the value of the field is then
+returned.
+
+   has $field :accessor;
+
+   # equivalent to
+   has $field;
+
+   method field {
+      $field = shift if @_;
+      return $field;
+   }
+
+=head3 :weak
+
+I<Since version 0.44.>
+
+Generated code which sets the value of this field will weaken it if it
+contains a reference. This applies to within the constructor if C<:param> is
+given, and to a C<:writer> accessor method. Note that this I<only> applies to
+automatically generated code; not normal code written in regular method
+bodies. If you assign into the field variable you must remember to call
+C<Scalar::Util::weaken> yourself.
+
+=head3 :param, :param(NAME)
+
+I<Since version 0.41.>
+
+Sets this field to be initialised automatically in the generated constructor.
+This is only permitted on scalar fields. If no name is given, the name of the
+field is used. A single prefix character C<_> will be removed if present.
+
+Any field that has C<:param> but does not have a default initialisation
+expression or block becomes a required argument to the constructor. Attempting
+to invoke the constructor without a named argument for this will throw an
+exception. In order to make a parameter optional, make sure to give it a
+default expression - even if that expression is C<undef>:
+
+   has $x :param;          # this is required
+   has $z :param = undef;  # this is optional
+
+Any field that has a C<:param> and an initialisation block will only run the
+code in the block if required by the constructor. If a named parameter is
+passed to the constructor for this field, then its code block will not be
+executed.
+
+Values for fields are assigned by the constructor before any C<BUILD> blocks
+are invoked.
+
 =head2 method
 
    method NAME {
@@ -369,11 +609,18 @@ case of a C<:writer> method.
       ...
    }
 
+   method NAME;
+
 Declares a new named method. This behaves similarly to the C<sub> keyword,
-except that within the body of the method all of the member fields ("slots")
-are also accessible. In addition, the method body will have a lexical called
-C<$self> which contains the invocant object directly; it will already have
-been shifted from the C<@_> array.
+except that within the body of the method all of the member fields are also
+accessible. In addition, the method body will have a lexical called C<$self>
+which contains the invocant object directly; it will already have been shifted
+from the C<@_> array.
+
+If the method has no body and is given simply as a name, this declares a
+I<required> method for a role. Such a method must be provided by any class
+that implements the role. It will be a compiletime error to combine the role
+with a class that does not provide this.
 
 The C<signatures> feature is automatically enabled for method declarations. In
 this case the signature does not have to account for the invocant instance; 
@@ -387,8 +634,8 @@ that is handled directly.
    $obj->m(1, 2);
 
 A list of attributes may be supplied as for C<sub>. The most useful of these
-is C<:lvalue>, allowing easy creation of read-write accessors for slots (but
-see also the C<:reader>, C<:writer> and C<:mutator> slot attributes).
+is C<:lvalue>, allowing easy creation of read-write accessors for fields (but
+see also the C<:reader>, C<:writer> and C<:mutator> field attributes).
 
    class Counter {
       has $count;
@@ -413,6 +660,48 @@ Marks that this method expects to override another of the same name from a
 superclass. It is an error at compiletime if the superclass does not provide
 such a method.
 
+=head2 method (lexical)
+
+   method $var { ... }
+
+   method $var :ATTRS... (SIGNATURE) { ... }
+
+I<Since version 0.59.>
+
+Declares a new lexical method. Lexical methods are not visible via the package
+namespace, but instead are stored directly in a lexical variable (with the
+same scoping rules as regular C<my> variables). These can be invoked by
+subsequent method code in the same block by using C<< $self->$var(...) >>
+method call syntax.
+
+   class WithPrivate {
+      has $var;
+
+      # Lexical methods can still see instance fields as normal
+      method $inc_var { $var++; say "Var was incremented"; }
+      method $dec_var { $var--; say "Var was decremented"; }
+
+      method bump {
+         $self->$inc_var;
+         say "In the middle";
+         $self->$dec_var;
+      }
+   }
+
+   my $obj = WithPrivate->new;
+
+   $obj->bump;
+
+   # Neither $inc_var nor $dec_var are visible here
+
+This effectively provides the ability to define B<private> methods, as they
+are inaccessible from outside the block that defines the class. In addition,
+there is no chance of a name collision because lexical variables in different
+scopes are independent, even if they share the same name. This is particularly
+useful in roles, to create internal helper methods without letting those
+methods be visible to callers, or risking their names colliding with other
+named methods defined on the consuming class.
+
 =head2 BUILD
 
    BUILD {
@@ -430,10 +719,52 @@ subroutine signature syntax, as for methods, to assist in unpacking its
 arguments. A build block is not a subroutine and thus is not permitted to use
 subroutine attributes (for example C<:lvalue>).
 
-Currently attempts to create a method named C<BUILD> (i.e. with syntax
-C<method BUILD {...}>) will create a builder block instead. As of version 0.31
-such attempts will print a warning at compiletime, and a later version may
-remove this altogether.
+Note that a C<BUILD> block is a named phaser block and not a method. Attempts
+to create a method named C<BUILD> (i.e. with syntax C<method BUILD {...}>)
+will fail with a compiletime error, to avoid this confusion.
+
+=head2 ADJUST
+
+   ADJUST {
+      ...
+   }
+
+I<Since version 0.43.>
+
+Declares an adjust block for this component class. This block of code runs
+within the constructor, after any C<BUILD> blocks and automatic field value
+assignment. It can make any final adjustments to the instance (such as
+initialising fields from calculated values). No additional parameters are
+passed.
+
+An adjust block is not a subroutine and thus is not permitted to use
+subroutine attributes. Note that an C<ADJUST> block is a named phaser block
+and not a method; it does not use the C<sub> or C<method> keyword.
+
+=head2 ADJUSTPARAMS
+
+   ADJUSTPARAMS ( $params ) {    # on perl 5.26 onwards
+      ...
+   }
+
+   ADJUSTPARAMS {
+      my $params = shift;
+      ...
+   }
+
+I<Since version 0.51.>
+
+Declares an adjust block for this component class that receives the parameters
+hash reference. This block of code runs within the constructor at the same
+time as L</ADJUST> blocks, but receives in addition a reference to the hash
+containing the current constructor parameters. This hash will not contain any
+constructor parameters already consumed by L</:param> declarations on any
+fields, but only the leftovers once those are processed.
+
+The code in the block should C<delete> from this hash any parameters it wishes
+to consume. Once all the C<ADJUSTPARAMS> blocks have run, any remaining keys
+in the hash will be considered errors, subject to the L</:strict(params)>
+check.
 
 =head2 requires
 
@@ -443,7 +774,16 @@ Declares that this role requires a method of the given name from any class
 that implements it. It is an error at compiletime if the implementing class
 does not provide such a method.
 
-=head1 IMPLIED PRAGMATA
+This form of declaring a required method is now vaguely discouraged, in favour
+of the bodyless C<method> form described above.
+
+=head1 CREPT FEATURES
+
+While not strictly part of being an object system, this module has
+nevertheless gained a number of behaviours by feature creep, as they have been
+found useful.
+
+=head2 Implied Pragmata
 
 In order to encourage users to write clean, modern code, the body of the
 C<class> block acts as if the following pragmata are in effect:
@@ -464,6 +804,12 @@ find any new restrictions to be majorly problematic. Either the code will
 continue to run unaffected, or you may have to make some small alterations to
 bring it into a conforming style.
 
+=head2 Yield True
+
+A C<class> statement or block will yield a true boolean value. This means that
+it can be used directly inside a F<.pm> file, avoiding the need to explicitly
+yield a true value from the end of it.
+
 =head1 SUBCLASSING CLASSIC PERL CLASSES
 
 There are a number of details specific to the case of deriving an
@@ -480,7 +826,7 @@ It is common in classic Perl OO style to invoke methods on C<$self> during
 the constructor. This is supported here since C<Object::Pad> version 0.19.
 Note however that any methods invoked by the superclass constructor may not
 see the object in a fully consistent state. (This fact is not specific to
-using C<Object::Pad> and would happen in classic Perl OO as well). The slot
+using C<Object::Pad> and would happen in classic Perl OO as well). The field
 initialisers will have been invoked but the C<BUILD> blocks will not.
 
 For example; in the following
@@ -494,7 +840,7 @@ For example; in the following
       sub get_value { return "A" }
    }
 
-   class DerivedClass extends ClassicPerlBaseClass {
+   class DerivedClass :isa(ClassicPerlBaseClass) {
       has $_value = "B";
       BUILD {
          $_value = "C";
@@ -506,7 +852,7 @@ For example; in the following
    say "Value seen by user is ", $obj->get_value;
 
 Until the C<ClassicPerlBaseClass::new> superconstructor has returned the
-C<BUILD> block will not have been invoked. The C<$_value> slot will still
+C<BUILD> block will not have been invoked. The C<$_value> field will still
 exist, but its value will be C<B> during the superconstructor. After the
 superconstructor, the C<BUILD> blocks are invoked before the completed object
 is returned to the user. The result will therefore be:
@@ -566,9 +912,9 @@ be implied by the C<class> keyword.
 
    # has, methods, etc.. can go here
 
-=head2 Slot Names
+=head2 Field Names
 
-Slot names should follow similar rules to regular lexical variables in code -
+Field names should follow similar rules to regular lexical variables in code -
 lowercase, name components separated by underscores. For tiny examples such as
 "dumb record" structures this may be sufficient.
 
@@ -578,10 +924,10 @@ lowercase, name components separated by underscores. For tiny examples such as
    }
 
 In larger examples with lots of non-trivial method bodies, it can get
-confusing to remember where the slot variables come from (because we no longer
-have the C<< $self->{ ... } >> visual clue). In these cases it is suggested to
-prefix the slot names with a leading underscore, to make them more visually
-distinct.
+confusing to remember where the field variables come from (because we no
+longer have the C<< $self->{ ... } >> visual clue). In these cases it is
+suggested to prefix the field names with a leading underscore, to make them
+more visually distinct.
 
    class Spudger {
       has $_grapefruit;
@@ -589,7 +935,7 @@ distinct.
       ...
 
       method mangle {
-         $_grapefruit->peel; # The leading underscore reminds us this is a slot
+         $_grapefruit->peel; # The leading underscore reminds us this is a field
       }
    }
 
@@ -616,14 +962,14 @@ sub import_into
    croak "Unrecognised import symbols @{[ keys %syms ]}" if keys %syms;
 }
 
-# Double-experimental MOP-like API
-# Currently entirely undocumented. May eventually become part of an Object::Pad::MOP API
 sub begin_class
 {
    my $class = shift;
    my ( $name, %args ) = @_;
 
-   Object::Pad::_begin_class( $name, $args{extends} );
+   Carp::carp "Object::Pad->begin_class is deprecated; use Object::Pad::MOP::Class->begin_class instead";
+
+   Object::Pad::MOP::Class->begin_class( $name, %args );
 }
 
 # The universal base-class methods
@@ -634,12 +980,20 @@ sub Object::Pad::UNIVERSAL::BUILDARGS
    return @_;
 }
 
+# Back-compat wrapper
+sub Object::Pad::MOP::SlotAttr::register
+{
+   shift; # $class
+   carp "Object::Pad::MOP::SlotAttr->register is now deprecated; use Object::Pad::MOP::FieldAttr->register instead";
+   return Object::Pad::MOP::FieldAttr->register( @_ );
+}
+
 =head1 WITH OTHER MODULES
 
 =head2 Syntax::Keyword::Dynamically
 
 A cross-module integration test asserts that C<dynamically> works correctly
-on object instance slots:
+on object instance fields:
 
    use Object::Pad;
    use Syntax::Keyword::Dynamically;
@@ -675,12 +1029,12 @@ async subs which await expressions:
    }
 
 These three modules combine; there is additionally a cross-module test to
-ensure that object instance slots can be C<dynamically> set during a suspended
-C<async method>.
+ensure that object instance fields can be C<dynamically> set during a
+suspended C<async method>.
 
 =head1 DESIGN TODOs
 
-The following points are details about the design of pad slot-based object
+The following points are details about the design of pad field-based object
 systems in general:
 
 =over 4
@@ -688,12 +1042,12 @@ systems in general:
 =item *
 
 Is multiple inheritence actually required, if role composition is implemented
-including giving roles the ability to use private slots?
+including giving roles the ability to use private fields?
 
 =item *
 
-Consider the visibility of superclass slots to subclasses. Do subclasses even
-need to be able to see their superclass's slots, or are accessor methods
+Consider the visibility of superclass fields to subclasses. Do subclasses
+even need to be able to see their superclass's fields, or are accessor methods
 always appropriate?
 
 Concrete example: The C<< $self->{split_at} >> access that
@@ -725,7 +1079,7 @@ Work out why we don't get a C<Subroutine new redefined at ...> warning if we
 
 =item *
 
-The C<local> modifier does not work on slot variables, because they appear to
+The C<local> modifier does not work on field variables, because they appear to
 be regular lexicals to the parser at that point. A workaround is to use
 L<Syntax::Keyword::Dynamically> instead:
 
@@ -760,6 +1114,34 @@ The C<#cor> IRC channel on C<irc.perl.org>.
 
 =back
 
+=cut
+
+=head1 SPONSORS
+
+With thanks to the following sponsors, who have helped me be able to spend
+time working on this module and other perl features.
+
+=over 4
+
+=item *
+
+Oetiker+Partner AG L<https://www.oetiker.ch/en/>
+
+=item *
+
+Deriv L<http://deriv.com>
+
+=item *
+
+Perl-Verein Schweiz L<https://www.perl-workshop.ch/>
+
+=back
+
+Additional details may be found at
+L<https://github.com/Ovid/Cor/wiki/Sponsors>.
+
+=cut
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
